@@ -34,15 +34,17 @@ from distutils.log import info
 try:
     import distro
 except ImportError:
-    python_version = tuple(int(x) for x in platform.python_version_tuple())
+    python_version = tuple(int(x) for x in platform.python_version_tuple()[:2])
     if python_version >= (3, 8):
         print(
             'Missing build requirement "distro" Python module; '
             'install paths may be incorrect', file=sys.stderr)
 
+windows_build = os.name == 'nt'
+
 
 def has_help(self):
-    return "build_help" in self.distribution.cmdclass and os.name != 'nt'
+    return "build_help" in self.distribution.cmdclass and not windows_build
 
 
 def has_icons(self):
@@ -50,7 +52,7 @@ def has_icons(self):
 
 
 def has_i18n(self):
-    return "build_i18n" in self.distribution.cmdclass and os.name != 'nt'
+    return "build_i18n" in self.distribution.cmdclass and not windows_build
 
 
 def has_data(self):
@@ -89,6 +91,14 @@ class build_data(distutils.cmd.Command):
 
     win32_settings_ini = '[Settings]\ngtk-application-prefer-dark-theme=0\n'
 
+    style_source = "data/styles/*.style-scheme.xml.in"
+    style_target_dir = 'share/meld/styles'
+
+    # FIXME: This is way too much hard coding, but I really hope
+    # it also doesn't last that long.
+    resource_source = "meld/resources/meld.gresource.xml"
+    resource_target = "org.gnome.Meld.gresource"
+
     def initialize_options(self):
         pass
 
@@ -96,7 +106,26 @@ class build_data(distutils.cmd.Command):
         pass
 
     def get_data_files(self):
-        if os.name == 'nt':
+        data_files = []
+
+        build_path = os.path.join('build', 'data')
+        if not os.path.exists(build_path):
+            os.makedirs(build_path)
+
+        info("compiling gresources")
+        resource_dir = os.path.dirname(self.resource_source)
+        target = os.path.join(build_path, self.resource_target)
+        self.spawn([
+            "glib-compile-resources",
+            "--target={}".format(target),
+            "--sourcedir={}".format(resource_dir),
+            "--sourcedir={}".format("data/icons/hicolor"),
+            self.resource_source,
+        ])
+
+        data_files.append(('share/meld', [target]))
+
+        if windows_build:
             # Write out a default settings.ini for Windows to make
             # e.g., dark theme selection slightly easier.
             settings_dir = os.path.join('build', 'etc', 'gtk-3.0')
@@ -106,11 +135,29 @@ class build_data(distutils.cmd.Command):
             with open(settings_path, 'w') as f:
                 print(self.win32_settings_ini, file=f)
 
-            return self.frozen_gschemas + [
+            gschemas = self.frozen_gschemas + [
                 ('etc/gtk-3.0', [settings_path])
             ]
         else:
-            return self.gschemas
+            gschemas = self.gschemas
+        data_files.extend(gschemas)
+
+        if windows_build:
+            # These should get moved/installed by i18n, but until that
+            # runs on Windows we need this hack.
+            styles = glob.glob(self.style_source)
+
+            import shutil
+            targets = []
+            for style in styles:
+                assert style.endswith('.in')
+                target = style[:-len('.in')]
+                shutil.copyfile(style, target)
+                targets.append(target)
+
+            data_files.append((self.style_target_dir, targets))
+
+        return data_files
 
     def run(self):
         data_files = self.distribution.data_files
@@ -218,7 +265,7 @@ class build_icons(distutils.cmd.Command):
         pass
 
     def run(self):
-        target_dir = self.frozen_target if os.name == 'nt' else self.target
+        target_dir = self.frozen_target if windows_build else self.target
         data_files = self.distribution.data_files
 
         for theme in glob.glob(os.path.join(self.icon_dir, "*")):
@@ -249,6 +296,7 @@ class build_i18n(distutils.cmd.Command):
     # way except magically extracting them from self.distribution.data_files
     desktop_files = [('share/applications', glob.glob("data/*.desktop.in"))]
     xml_files = [
+        ('share/meld/styles', glob.glob("data/styles/*.style-scheme.xml.in")),
         ('share/metainfo', glob.glob("data/*.appdata.xml.in")),
         ('share/mime/packages', glob.glob("data/mime/*.xml.in"))
     ]
@@ -273,7 +321,7 @@ class build_i18n(distutils.cmd.Command):
 
         # If we're on Windows, assume we're building frozen and make a bunch
         # of insane assumptions.
-        if os.name == 'nt':
+        if windows_build:
             msgfmt = "C:\\Python27\\Tools\\i18n\\msgfmt"
         else:
             msgfmt = "msgfmt"
@@ -315,6 +363,18 @@ class build_i18n(distutils.cmd.Command):
             os.environ["XGETTEXT_ARGS"] = "--msgid-bugs-address=%s " % \
                                           self.bug_contact
 
+        # These copies are pure hacks to work around not having the
+        # Meson-based initial variable templating in distutils.
+        import shutil
+        shutil.copyfile(
+            'data/org.gnome.meld.desktop.in.in',
+            'data/org.gnome.meld.desktop.in',
+        )
+        shutil.copyfile(
+            'data/org.gnome.meld.appdata.xml.in.in',
+            'data/org.gnome.meld.appdata.xml.in',
+        )
+
         self._rebuild_po()
 
         intltool_switches = [
