@@ -17,12 +17,10 @@
 
 import math
 
-from gi.repository import Gdk
-from gi.repository import Gtk
-
-from meld.misc import get_common_theme
-from meld.settings import meldsettings
+from gi.repository import Gdk, Gtk
 
+from meld.settings import get_meld_settings
+from meld.style import get_common_theme
 
 # Rounded rectangle corner radius for culled changes display
 RADIUS = 3
@@ -34,7 +32,7 @@ class LinkMap(Gtk.DrawingArea):
 
     def __init__(self):
         self.filediff = None
-        meldsettings.connect('changed', self.on_setting_changed)
+        self.views = []
 
     def associate(self, filediff, left_view, right_view):
         self.filediff = filediff
@@ -43,14 +41,16 @@ class LinkMap(Gtk.DrawingArea):
             self.views.reverse()
         self.view_indices = [filediff.textview.index(t) for t in self.views]
 
-        self.on_setting_changed(meldsettings, 'style-scheme')
+        meld_settings = get_meld_settings()
+        self.on_setting_changed(meld_settings, 'style-scheme')
+        meld_settings.connect('changed', self.on_setting_changed)
 
     def on_setting_changed(self, settings, key):
         if key == 'style-scheme':
             self.fill_colors, self.line_colors = get_common_theme()
 
     def do_draw(self, context):
-        if not self.filediff:
+        if not self.views:
             return
 
         pix_start = [t.get_visible_rect().y for t in self.views]
@@ -67,19 +67,23 @@ class LinkMap(Gtk.DrawingArea):
         context.set_line_width(1.0)
 
         height = allocation.height
-        visible = [self.views[0].get_line_num_for_y(pix_start[0]),
-                   self.views[0].get_line_num_for_y(pix_start[0] + height),
-                   self.views[1].get_line_num_for_y(pix_start[1]),
-                   self.views[1].get_line_num_for_y(pix_start[1] + height)]
+        visible = [
+            self.views[0].get_line_num_for_y(pix_start[0]),
+            self.views[0].get_line_num_for_y(pix_start[0] + height),
+            self.views[1].get_line_num_for_y(pix_start[1]),
+            self.views[1].get_line_num_for_y(pix_start[1] + height),
+        ]
 
-        wtotal = allocation.width
         # For bezier control points
-        x_steps = [-0.5, wtotal / 2, wtotal / 2, wtotal + 0.5]
+        x_steps = [-0.5, allocation.width / 2, allocation.width + 0.5]
         q_rad = math.pi / 2
 
         left, right = self.view_indices
-        view_offset_line = lambda v, l: (self.views[v].get_y_for_line_num(l) -
-                                         pix_start[v] + y_offset[v])
+
+        def view_offset_line(view_idx, line_num):
+            line_start = self.views[view_idx].get_y_for_line_num(line_num)
+            return line_start - pix_start[view_idx] + y_offset[view_idx]
+
         for c in self.filediff.linediffer.pair_changes(left, right, visible):
             # f and t are short for "from" and "to"
             f0, f1 = [view_offset_line(0, l) for l in c[1:3]]
@@ -92,24 +96,25 @@ class LinkMap(Gtk.DrawingArea):
             if (t0 < 0 and t1 < 0) or (t0 > height and t1 > height):
                 if f0 == f1:
                     continue
-                context.arc(x_steps[0], f0 - 0.5 + RADIUS, RADIUS, -q_rad, 0)
+                context.arc(
+                    x_steps[0], f0 - 0.5 + RADIUS, RADIUS, q_rad * 3, 0)
                 context.arc(x_steps[0], f1 - 0.5 - RADIUS, RADIUS, 0, q_rad)
                 context.close_path()
             elif (f0 < 0 and f1 < 0) or (f0 > height and f1 > height):
                 if t0 == t1:
                     continue
-                context.arc_negative(x_steps[3], t0 - 0.5 + RADIUS, RADIUS,
-                                     -q_rad, q_rad * 2)
-                context.arc_negative(x_steps[3], t1 - 0.5 - RADIUS, RADIUS,
+                context.arc_negative(x_steps[2], t0 - 0.5 + RADIUS, RADIUS,
+                                     q_rad * 3, q_rad * 2)
+                context.arc_negative(x_steps[2], t1 - 0.5 - RADIUS, RADIUS,
                                      q_rad * 2, q_rad)
                 context.close_path()
             else:
                 context.move_to(x_steps[0], f0 - 0.5)
                 context.curve_to(x_steps[1], f0 - 0.5,
-                                 x_steps[2], t0 - 0.5,
-                                 x_steps[3], t0 - 0.5)
-                context.line_to(x_steps[3], t1 - 0.5)
-                context.curve_to(x_steps[2], t1 - 0.5,
+                                 x_steps[1], t0 - 0.5,
+                                 x_steps[2], t0 - 0.5)
+                context.line_to(x_steps[2], t1 - 0.5)
+                context.curve_to(x_steps[1], t1 - 0.5,
                                  x_steps[1], f1 - 0.5,
                                  x_steps[0], f1 - 0.5)
                 context.close_path()
@@ -126,27 +131,10 @@ class LinkMap(Gtk.DrawingArea):
             Gdk.cairo_set_source_rgba(context, self.line_colors[c[0]])
             context.stroke()
 
-    def do_scroll_event(self, event):
-        self.filediff.next_diff(event.direction)
 
-try:
-    LinkMap.set_css_name("link-map")
-except AttributeError:
-    # New API in 3.20
-    pass
+LinkMap.set_css_name("link-map")
 
 
 class ScrollLinkMap(Gtk.DrawingArea):
 
     __gtype_name__ = "ScrollLinkMap"
-
-    def __init__(self):
-        self.melddoc = None
-
-    def associate(self, melddoc):
-        self.melddoc = melddoc
-
-    def do_scroll_event(self, event):
-        if not self.melddoc:
-            return
-        self.melddoc.next_diff(event.direction)
