@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2002-2005 Stephen Kennedy <stevek@gnome.org>
 # Copyright (C) 2011-2013, 2015 Kai Willadsen <kai.willadsen@gmail.com>
 
@@ -26,14 +25,14 @@
 import errno
 import glob
 import os
-import shutil
-import tempfile
 import xml.etree.ElementTree as ElementTree
-import subprocess
 
 from meld.conf import _
 from . import _vc
 
+#: Simple enum constants for differentiating conflict cases.
+CONFLICT_TYPE_MERGE, CONFLICT_TYPE_UPDATE = 1, 2
+
 
 class Vc(_vc.Vc):
 
@@ -82,25 +81,16 @@ class Vc(_vc.Vc):
             raise _vc.InvalidVCPath(self, path, "Path not in repository")
         path = path[len(self.root) + 1:]
 
-        process = subprocess.Popen([self.CMD, "cat", "-r", commit, path],
-                                   cwd=self.root, stdout=subprocess.PIPE,
-                                   stderr=subprocess.PIPE)
-        vc_file = process.stdout
-
-        # Error handling here involves doing nothing; in most cases, the only
-        # sane response is to return an empty temp file. The most common error
-        # is "no base revision until committed" from diffing a new file.
-
-        with tempfile.NamedTemporaryFile(prefix='meld-tmp', delete=False) as f:
-            shutil.copyfileobj(vc_file, f)
-        return f.name
+        suffix = os.path.splitext(path)[1]
+        args = [self.CMD, "cat", "-r", commit, path]
+        return _vc.call_temp_output(args, cwd=self.root, suffix=suffix)
 
     def get_path_for_conflict(self, path, conflict=None):
         """
         SVN has two types of conflicts:
         Merge conflicts, which give 3 files:
            .left.r* (THIS)
-           .working (BASE... although this is a bit debateable)
+           .working (BASE... although this is a bit debatable)
            .right.r* (OTHER)
         Update conflicts which give 3 files:
            .mine (THIS)
@@ -114,9 +104,6 @@ class Vc(_vc.Vc):
         if conflict == _vc.CONFLICT_MERGED:
             return path, False
 
-        CONFLICT_TYPE_MERGE = 1
-        CONFLICT_TYPE_UPDATE = 2
-
         # First fine what type of conflict this is by looking at the base
         # we can possibly return straight away!
         conflict_type = None
@@ -188,9 +175,10 @@ class Vc(_vc.Vc):
     def _update_tree_state_cache(self, path):
         while 1:
             try:
+                # "svn --xml" outputs utf8, even with Windows non-utf8 locale
                 proc = _vc.popen(
                     [self.CMD, "status", "-v", "--xml", path],
-                    cwd=self.location)
+                    cwd=self.location, use_locale_encoding=False)
                 tree = ElementTree.parse(proc)
                 break
             except OSError as e:
@@ -198,14 +186,13 @@ class Vc(_vc.Vc):
                     raise
 
         for target in tree.findall("target") + tree.findall("changelist"):
-            for entry in (t for t in target.getchildren() if t.tag == "entry"):
+            for entry in target.iter(tag="entry"):
                 path = entry.attrib["path"]
                 if not path:
                     continue
                 if not os.path.isabs(path):
                     path = os.path.abspath(os.path.join(self.location, path))
-                for status in (e for e in entry.getchildren()
-                               if e.tag == "wc-status"):
+                for status in entry.iter(tag="wc-status"):
                     item = status.attrib["item"]
                     if item == "":
                         continue
@@ -215,3 +202,4 @@ class Vc(_vc.Vc):
                     rev = status.attrib.get("revision")
                     rev_label = _("Rev %s") % rev if rev is not None else ''
                     self._tree_meta_cache[path] = rev_label
+                    self._add_missing_cache_entry(path, state)
