@@ -1,8 +1,13 @@
+from unittest import mock
 
 import pytest
 from gi.repository import Gio
 
-from meld.iohelpers import find_shared_parent_path, format_parent_relative_path
+from meld.iohelpers import (
+    find_shared_parent_path,
+    format_home_relative_path,
+    format_parent_relative_path,
+)
 
 
 @pytest.mark.parametrize(
@@ -43,6 +48,23 @@ def test_find_shared_parent_path(paths, expected_parent):
             assert parent.equal(Gio.File.new_for_path(expected_parent))
 
 
+@pytest.mark.parametrize(
+    "path, expected_format",
+    [
+        ("/home/hey/foo", "~/foo"),
+        ("/home/hmph/foo", "/home/hmph/foo"),
+    ]
+)
+def test_format_home_relative_path(path, expected_format):
+
+    with mock.patch(
+        "meld.iohelpers.GLib.get_home_dir",
+        return_value="/home/hey/",
+    ):
+        gfile = Gio.File.new_for_path(path)
+        assert format_home_relative_path(gfile) == expected_format
+
+
 @pytest.mark.parametrize(
     'parent, child, expected_label',
     [
@@ -64,11 +86,25 @@ def test_find_shared_parent_path(paths, expected_parent):
             '/home/hey/project/foo.txt',
             '…/project/foo.txt',
         ),
-        # Child is a more-than-2-depth child of parent
+        # Child is a 3-depth child of parent
         (
             '/home/hey/',
-            '/home/hey/project/hey/hey/foo.txt',
-            '…/project/…/foo.txt',
+            '/home/hey/project/package/foo.txt',
+            '…/project/package/foo.txt',
+        ),
+        # Child is a more-than-3-depth child of parent, with long
+        # immediate parent
+        (
+            '/home/hey/',
+            '/home/hey/project/package/subpackage/foo.txt',
+            '…/project/…/subpackage/foo.txt',
+        ),
+        # Child is a more-than-3-depth child of parent, with short
+        # immediate parent
+        (
+            '/home/hey/',
+            '/home/hey/project/package/src/foo.txt',
+            '…/project/package/src/foo.txt',
         ),
     ],
 )
